/*
このライブラリの使い方は次の通りです。

    // 新しいクイズを生成
    const quiz = require('./outsider-quiz').create();

quizの中身は以下のようなオブジェクトです。

    {
      items: [ 'バナナ', 'テレビ', 'りんご' ],
      answerIndex: 1
    }
*/

'use strict';

// アイテム一覧
const fruits = ['りんご', 'バナナ', 'オレンジ'];
const sweets = ['ケーキ', 'クッキー', 'プリン'];
const machines = ['飛行機', 'テレビ', '冷蔵庫'];
const groups = [fruits, sweets, machines];
 
// 0から(num-1)までのランダムな整数を返す
function randInt(num) {
  return Math.floor(Math.random() * num);
}
 
// 配列arrの中からランダムに要素を返す
function getRandomItem(arr) {
  return arr[randInt(arr.length)];
}
 
// 配列arrをシャッフルする
// （Fisher-Yates-Durstenfeldアルゴリズム）
function shuffle(arr) {
  for (let i = arr.length - 1; i > 0; i--) {
    const j = randInt(i + 1);
 
    // arr[i] と arr[j] を入れ替える
    [ arr[j], arr[i] ] = [ arr[i], arr[j] ];
  }
}
 
// クイズを生成して返す
exports.create = function() {
  // ベースとするアイテムグループをランダムに選ぶ
  const baseGroupIndex = randInt(groups.length);
 
  // 選んだアイテムグループをコピー
  const options = groups[baseGroupIndex].slice(0);
 
  // 中身をシャッフル
  shuffle(options);
 
  // ベースとするアイテムグループ以外から
  // 仲間外れグループをランダムに選ぶ
  const outsiderGroupIndex = (
    baseGroupIndex + 1 + randInt(groups.length - 1)
  ) % groups.length;
 
  // 仲間外れグループからアイテムをランダムに選ぶ
  const outsiderGroup = groups[outsiderGroupIndex];
  const outsiderItem = getRandomItem(outsiderGroup);
 
  // optionsの中のどれかを仲間外れアイテムに置き換える
  const replaceIndex = randInt(options.length);
  options[replaceIndex] = outsiderItem;
 
  return {
    items: options, // 選択肢
    answerIndex: replaceIndex, // 何番目が正解か
  };
};
